@@ -558,6 +558,14 @@ implements JavaObjective, java.io.Serializable {
 		}
 	}
 
+	/* (non-Javadoc)
+	 * @see net.morilib.lisp.LispNumber#isFinite()
+	 */
+	@Override
+	public boolean isFinite() {
+		return true;
+	}
+
 	/*
 	 * (non-Javadoc)
 	 * @see java.lang.Object#equals(java.lang.Object)
