@@ -19,6 +19,7 @@ import java.lang.reflect.Array;
 import java.math.BigDecimal;
 import java.math.BigInteger;
 import java.util.ArrayList;
+import java.util.Collection;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.IdentityHashMap;
@@ -29,10 +30,13 @@ import java.util.Stack;
 import java.util.Map.Entry;
 import java.util.Set;
 
-import net.morilib.lisp.array.LispArray;
+import net.morilib.lisp.array.ILispArray;
+import net.morilib.lisp.datetime.LispDate;
 import net.morilib.lisp.uvector.HomogeneousArray;
 import net.morilib.util.ArrayListStack;
 import net.morilib.util.Stack2;
+import net.morilib.util.mapset.HashOneToOneSet;
+import net.morilib.util.mapset.OneToOneSet;
 import net.morilib.util.table.ArrayTable;
 import net.morilib.util.table.Table;
 
@@ -292,8 +296,8 @@ public final class LispUtils {
 		while(!st.isEmpty()) {
 			Datum d = st.pop();
 
-			if(d instanceof LispArray && !(d instanceof LispVector)) {
-				LispArray a = (LispArray)d;
+			if(d instanceof ILispArray && !(d instanceof LispVector)) {
+				ILispArray a = (ILispArray)d;
 				int[] is = new int[a.rank()];
 
 				buf.append('#');
@@ -420,6 +424,12 @@ public final class LispUtils {
 				buf.append(")");
 			} else if(d == _STK_MLT1) {
 				buf.append("\n");
+			} else if(d instanceof Datum3) {
+				if(met) {
+					((Datum3)d).getResult(buf);
+				} else {
+					((Datum3)d).print(buf);
+				}
 			} else {
 				d.toDisplayString(buf);
 			}
@@ -427,7 +437,7 @@ public final class LispUtils {
 	}
 
 	//
-	private static void _append(LispArray a, StringBuilder b,
+	private static void _append(ILispArray a, StringBuilder b,
 			Map<Datum, Object> eq, boolean met, boolean s38, int dp,
 			int[] is) {
 		if(dp == is.length) {
@@ -510,16 +520,20 @@ public final class LispUtils {
 	 * @param dot
 	 * @return
 	 */
-	public static Datum listToCons(List<Datum> d, Datum dot) {
+	public static Datum listToCons(
+			Collection<? extends Datum> d, Datum dot) {
 		Cons res = null;
 		Cons c2 = null;
 
-		for(int i = 0; i < d.size(); i++) {
+//		for(int i = 0; i < d.size(); i++) {
+		for(Datum x : d) {
 			if(res == null) {
 				res = c2 = new Cons();
-				c2.setCar(d.get(i));
+//				c2.setCar(d.get(i));
+				c2.setCar(x);
 			} else {
-				Cons c3 = new Cons(d.get(i), Nil.NIL);
+//				Cons c3 = new Cons(d.get(i), Nil.NIL);
+				Cons c3 = new Cons(x, Nil.NIL);
 				c2.setCdr(c3);
 				c2 = c3;
 			}
@@ -538,7 +552,7 @@ public final class LispUtils {
 	 * @param d
 	 * @return
 	 */
-	public static Datum listToCons(List<Datum> d) {
+	public static Datum listToCons(Collection<? extends Datum> d) {
 		return listToCons(d, Nil.NIL);
 	}
 
@@ -548,7 +562,8 @@ public final class LispUtils {
 	 * @param dot
 	 * @return
 	 */
-	public static Datum toCons(Iterable<Datum> d, Datum dot) {
+	public static Datum toCons(Iterable<? extends Datum> d,
+			Datum dot) {
 		Cons res = null;
 		Cons c2 = null;
 
@@ -576,7 +591,7 @@ public final class LispUtils {
 	 * @param d
 	 * @return
 	 */
-	public static Datum toCons(Iterable<Datum> d) {
+	public static Datum toCons(Iterable<? extends Datum> d) {
 		return toCons(d, Nil.NIL);
 	}
 
@@ -622,6 +637,16 @@ public final class LispUtils {
 		return res;
 	}
 
+	/**
+	 * 
+	 * @param d
+	 * @param mesg
+	 * @return
+	 */
+	public static Datum[] consToArray(Datum d, LispMessage mesg) {
+		return consToList(d, mesg).toArray(new Datum[0]);
+	}
+
 	/**
 	 * 
 	 * @param d
@@ -799,6 +824,10 @@ public final class LispUtils {
 			return JavaNull.JAVA_NULL;
 		} if(o instanceof Datum) {
 			return (Datum)o;
+		} else if(o instanceof Byte) {
+			return LispInteger.valueOf(((Byte)o).intValue());
+		} else if(o instanceof Short) {
+			return LispInteger.valueOf(((Short)o).intValue());
 		} else if(o instanceof Integer) {
 			return LispInteger.valueOf(((Integer)o).intValue());
 		} else if(o instanceof Long) {
@@ -1124,9 +1153,9 @@ public final class LispUtils {
 			} else if(x instanceof HomogeneousArray) {
 				return ((y instanceof HomogeneousArray) &&
 						((HomogeneousArray)x).equalsArray(x, y));
-			} else if(x instanceof LispArray) {
-				return ((y instanceof LispArray) &&
-						((LispArray)x).isEqualTo((LispArray)y));
+			} else if(x instanceof ILispArray) {
+				return ((y instanceof ILispArray) &&
+						((ILispArray)x).isEqualTo((ILispArray)y));
 			} else if(!x.equals(y)) {
 				return false;
 			}
@@ -1392,7 +1421,7 @@ public final class LispUtils {
 		Map<Datum, Datum> eq = new IdentityHashMap<Datum, Datum>();
 		long l = 0;
 
-		while(dd != Nil.NIL) {
+		while(!dd.isNil()) {
 			if(eq.containsKey(dd)) {
 				return -1;
 			} else if(dd instanceof Cons) {
@@ -1406,4 +1435,384 @@ public final class LispUtils {
 		return l;
 	}
 
+	/**
+	 * 
+	 * @param x
+	 * @return
+	 */
+	public static boolean isProperList(Datum x) {
+		return length(x) >= 0;
+	}
+
+	/**
+	 * 
+	 * @param x
+	 * @return
+	 */
+	public static boolean isProperListNotNil(Datum x) {
+		return length(x) > 0;
+	}
+
+	/**
+	 * 
+	 * @param x
+	 * @param n
+	 * @param mesg
+	 * @return
+	 */
+	public static Datum take(Datum x, int n) {
+		Datum x1 = x;
+		Cons  r0, rp;
+
+		r0 = rp = null;
+		for(int i = 0; i < n; i++) {
+			if(x1 instanceof Cons) {
+				if(r0 == null) {
+					rp = new Cons();
+					r0 = rp;
+				} else {
+					rp.setCdr(new Cons());
+					rp = (Cons)rp.getCdr();
+				}
+				rp.setCar(((Cons)x1).getCar());
+				rp.setCdr(Nil.NIL);
+				x1 = ((Cons)x1).getCdr();
+			} else {
+				if(r0 == null) {
+					return x1;
+				} else {
+					rp.setCdr(x1);
+					return r0;
+				}
+			}
+		}
+		return (r0 != null) ? r0 : Nil.NIL;
+	}
+
+	/**
+	 * 
+	 * @param x
+	 * @param y
+	 * @return
+	 */
+	public static Datum mul(Datum x, Datum y) {
+		return (x.isNil() || y.isNil()) ? Nil.NIL : new Cons(x, y);
+	}
+
+	/**
+	 * 
+	 * @param x
+	 * @return
+	 */
+	public static boolean isProduct(Datum x) {
+		Stack2<Datum> s = new ArrayListStack<Datum>();
+		Set<Datum>    t = new HashSet<Datum>();
+		Datum d;
+
+		s.push(x);
+		while(s.isEmpty()) {
+			d = s.pop();
+			if(t.contains(d)) {
+				// has common reference
+				return false;
+			} else if(d instanceof Cons) {
+				s.push(((Cons)d).getCdr());
+				s.push(((Cons)d).getCar());
+				t.add(d);
+			} else if(d.isNil()) {
+				return false;
+			}
+		}
+		return true;
+	}
+
+	/**
+	 * 
+	 * @param x
+	 * @param y
+	 * @return
+	 */
+	public static Datum append(Datum x, Datum y) {
+		ConsListBuilder b = new ConsListBuilder();
+		ConsIterator itr = new ConsIterator(x);
+
+		if(!(x instanceof Cons) || !(y instanceof Cons)) {
+			return null;
+		} else if(x.isNilUnit()) {
+			return y;
+		} else if(y.isNilUnit()) {
+			return x;
+		} else {
+			while(itr.hasNext()) {
+				b.append(itr.next());
+			}
+			return itr.getTerminal().isNil() ? b.get(y) : null;
+		}
+	}
+
+	/**
+	 * 
+	 * @param x
+	 * @param y
+	 * @return
+	 */
+	public static Datum add(Datum x, Datum y) {
+		Datum lx, ly;
+
+		if(x.isNil()) {
+			return y;
+		} else if(y.isNil()) {
+			return x;
+		}
+		lx = isProperList(x) ? x : list(x);
+		ly = isProperList(y) ? y : list(y);
+		return append(lx, ly);
+	}
+
+	/**
+	 * 
+	 * @param x
+	 * @param y
+	 * @return
+	 */
+	public static Procedure mul(final Procedure x, final Procedure y) {
+		return new Subr("synthesized procedure") {
+
+			@Override
+			public Datum eval(Datum body, Environment env,
+					LispMessage mesg) {
+				Datum ry = Scheme.call(y, env, mesg, body);
+				Datum[] ay;
+
+				ay = ry.getValues().toArray(new Datum[0]);
+				return Scheme.callva(x, env, mesg, ay);
+			}
+
+		};
+	}
+
+	//
+	private static class LabeledCons extends Datum {
+
+		//
+		private long  label;
+		//private Datum cons;
+
+		//
+		private LabeledCons(Datum c, long l) {
+			this.label = l;
+			//this.cons  = c;
+		}
+
+		//
+		public int hashCode() {
+			return (int)label;
+		}
+
+		//
+		public boolean equals(Object o) {
+			if(o instanceof LabeledCons) {
+				return label == ((LabeledCons)o).label;
+			}
+			return false;
+		}
+	}
+
+	/**
+	 * 
+	 * @param d
+	 * @return
+	 */
+	public static List<Datum> serializeCons(Datum d) {
+		List<Datum> r;
+		Stack2<Datum> s1;
+		OneToOneSet<Long, Datum> sc;
+		long l = 0, m;
+		Datum x;
+
+		r  = new ArrayList<Datum>();
+		s1 = new ArrayListStack<Datum>();
+		sc = new HashOneToOneSet<Long, Datum>();
+		s1.add(d);
+		while(!s1.isEmpty()) {
+			x = s1.pop();
+			if(sc.containsValue(x)) {
+				m = sc.getKey(x);
+				r.add(new LabeledCons(x, l - m));
+			} else {
+				if(x instanceof Cons) {
+					sc.put(l++, x);
+					r.add(new LabeledCons(x, -1));
+					s1.push(((Cons)x).getCdr());
+					s1.push(((Cons)x).getCar());
+				} else {
+					r.add(x);
+				}
+			}
+		}
+		return r;
+	}
+
+	/**
+	 * 
+	 * @param a
+	 * @param b
+	 * @return
+	 */
+	public static boolean containsTopologically(Datum a, Datum b) {
+		List<Datum> a1, b1;
+
+		a1 = serializeCons(a);
+		b1 = serializeCons(b);
+		outer: for(int i = 0; i <= a1.size() - b1.size(); i++) {
+			for(int j = i; j < b1.size(); j++) {
+				if(!a1.get(j).equals(b1.get(j))) {
+					continue outer;
+				}
+			}
+			return true;
+		}
+		return false;
+	}
+
+	/**
+	 * 
+	 * @param l
+	 * @param d
+	 * @return
+	 */
+	public static boolean contains(Datum l, Datum d) {
+		Stack2<Datum> s1 = new ArrayListStack<Datum>();
+		Set<Datum> sc = new HashSet<Datum>();
+		Datum x;
+
+		s1.add(l);
+		while(!s1.isEmpty()) {
+			x = s1.pop();
+			if(x.equals(d)) {
+				return true;
+			} else if(sc.contains(x)) {
+				// do nothing
+			} else if(x instanceof Cons) {
+				sc.add(x);
+				s1.push(((Cons)x).getCdr());
+				s1.push(((Cons)x).getCar());
+			}
+		}
+		return false;
+	}
+
+	/**
+	 * 
+	 * @param l
+	 * @param d
+	 * @return
+	 */
+	public static int countLeaves(Datum d) {
+		Stack2<Datum> s1 = new ArrayListStack<Datum>();
+		Set<Datum> sc = new HashSet<Datum>();
+		Datum x;
+		int l = 0;
+
+		s1.add(d);
+		while(!s1.isEmpty()) {
+			x = s1.pop();
+			if(sc.contains(x)) {
+				return -1;
+			} else if(x instanceof Cons) {
+				sc.add(x);
+				s1.push(((Cons)x).getCdr());
+				s1.push(((Cons)x).getCar());
+			} else if(!x.isNil()) {
+				l++;
+			}
+		}
+		return l;
+	}
+
+	/**
+	 * 
+	 * @param d
+	 * @param mesg
+	 * @return
+	 */
+	public static Object toObject(Datum d, LispMessage mesg) {
+		if(d instanceof JavaObjective) {
+			return ((JavaObjective)d).toObject();
+		} else {
+			throw mesg.getError("err.require.java-instance", d);
+		}
+	}
+
+	/**
+	 * 
+	 * @param s
+	 * @return
+	 */
+	public static Datum toLispString(String s) {
+		return (s != null) ? new LispString(s) : LispBoolean.FALSE;
+	}
+
+	/**
+	 * 
+	 * @param s
+	 * @return
+	 */
+	public static Datum toPositiveInt(int x) {
+		return (x >= 0) ? LispInteger.valueOf(x) : LispBoolean.FALSE;
+	}
+
+	/**
+	 * 
+	 * @param s
+	 * @return
+	 */
+	public static Datum toDate(long x) {
+		return (x != 0) ? new LispDate(x) : LispBoolean.FALSE;
+	}
+
+	/**
+	 * @param charAt
+	 * @return
+	 */
+	public static boolean isReservedCharacter(char c) {
+		return (c == '(' || c == '[' ||
+				c == ')' || c == ']' || c == '#' ||
+				Character.isWhitespace(c));
+	}
+
+	/**
+	 * 
+	 * @param number
+	 * @return
+	 */
+	public static BigInteger toIntegerExact(double number) {
+		try {
+			BigDecimal dec;
+	
+			if(Double.isInfinite(number) || Double.isNaN(number)) {
+				return null;
+			}
+			dec = new BigDecimal(number);
+			return dec.toBigIntegerExact();
+		} catch(ArithmeticException e) {
+			return null;
+		}
+	}
+
+	/**
+	 * 
+	 * @param i
+	 * @return
+	 */
+	public static Integer toIntExact(BigInteger i) {
+		int r = i.intValue();
+	
+		if(i.equals(BigInteger.valueOf(r))) {
+			return new Integer(r);
+		} else {
+			return null;
+		}
+	}
+
 }
