@@ -20,6 +20,7 @@ import java.util.Iterator;
 import java.util.Map.Entry;
 import java.util.NoSuchElementException;
 
+import net.morilib.lisp.accessor.ILispRef;
 import net.morilib.lisp.collection.LispBag;
 import net.morilib.lisp.collection.LispCollection;
 import net.morilib.lisp.collection.LispDictionary;
@@ -29,6 +30,7 @@ import net.morilib.lisp.iterator.ILispIterable;
 import net.morilib.lisp.iterator.ILispIterator;
 import net.morilib.lisp.sos.LispType;
 import net.morilib.lisp.subr.IsEqual;
+import net.morilib.lisp.subr.SubrUtils;
 import net.morilib.util.Iterators;
 
 /**
@@ -38,7 +40,7 @@ import net.morilib.util.Iterators;
  * @author MORIGUCHI, Yuichiro 2009
  */
 public final class Nil extends SExpressionDatum
-implements ConsOrNil, ILispIterable, ILispIterator,
+implements ConsOrNil, ILispIterable, ILispIterator, ILispRef,
 java.io.Serializable {
 
 	/**
@@ -746,4 +748,13 @@ java.io.Serializable {
 		return this;
 	}
 
+	/* (non-Javadoc)
+	 * @see net.morilib.lisp.accessor.ILispRef#ref(net.morilib.lisp.Datum, net.morilib.lisp.LispMessage)
+	 */
+	@Override
+	public Datum ref(Datum arg, LispMessage mesg) {
+		SubrUtils.getSmallInt(arg, mesg);
+		throw mesg.getError("err.list.outofrange", arg);
+	}
+
 }
