@@ -150,6 +150,34 @@ public final class Scheme {
 		return newInstance(Locale.getDefault());
 	}
 
+	/**
+	 * 
+	 * @param lc
+	 * @return
+	 * @throws InitLoadException 
+	 */
+	public static Scheme newInstance(
+			String[] pkg, Locale lc) throws InitLoadException {
+		Scheme res = new Scheme(LispMessage.getInstance(lc));
+
+		for(String s : pkg)  InitSubrLoader.load(s, res.global);
+		InitVarLoader.load(res.global);
+		res.parser.clear();
+		InitLispLoader.load(res);
+		res.parser.clear();
+		return res;
+	}
+
+	/**
+	 * 
+	 * @return
+	 * @throws InitLoadException 
+	 */
+	public static Scheme newInstance(
+			String[] pkg) throws InitLoadException {
+		return newInstance(pkg, Locale.getDefault());
+	}
+
 	/**
 	 * 
 	 * @param ver
@@ -199,7 +227,17 @@ public final class Scheme {
 	 * @return
 	 */
 	public static Scheme newNull(int ver) {
-		Scheme res = new Scheme(LispMessage.getInstance());
+		return newNull(ver, null);
+	}
+
+	/**
+	 * 
+	 * @param ver
+	 * @param lc
+	 * @return
+	 */
+	public static Scheme newNull(int ver, Locale lc) {
+		Scheme res = new Scheme(LispMessage.getInstance(lc));
 
 		InitSubrLoader.loadNullEnv(res.global, ver);
 		InitVarLoader.loadNullEnv(res.global, ver);
@@ -215,7 +253,17 @@ public final class Scheme {
 	 * @return
 	 */
 	public static Scheme newRnRS(int ver) {
-		Scheme res = new Scheme(LispMessage.getInstance());
+		return newRnRS(ver, null);
+	}
+
+	/**
+	 * 
+	 * @param ver
+	 * @param lc
+	 * @return
+	 */
+	public static Scheme newRnRS(int ver, Locale lc) {
+		Scheme res = new Scheme(LispMessage.getInstance(lc));
 
 		InitSubrLoader.loadRnRSEnv(res.global, ver);
 		InitVarLoader.loadRnRSEnv(res.global, ver);
@@ -480,8 +528,9 @@ public final class Scheme {
 	 * @throws IOException
 	 */
 	public static void main(String[] args) throws IOException {
-		Scheme eval = Scheme.newInstance();
+		Scheme eval;
 		List<Datum> d1 = new ArrayList<Datum>();
+		int opt;
 
 		d1.add(new LispString("schluessel"));
 		for(String s : args) {
@@ -489,16 +538,19 @@ public final class Scheme {
 		}
 		CommandLine.commandLine = LispUtils.listToCons(d1);
 
-		if(args.length == 0) {
+//		eval = Scheme.newInstance();
+		eval = SchemeOptions.preparseOption(args);
+		opt  = SchemeOptions.parseOption(args, eval);
+		if(args.length == opt) {
 			eval.readEvalPrintLoop(new InputStreamReader(System.in));
 			System.exit(0);
 		} else {
 			InputStream ins;
 
-			if(args[0].equals("-")) {
+			if(args[opt].equals("-")) {
 				ins = System.in;
 			} else {
-				ins = new FileInputStream(args[0]);
+				ins = new FileInputStream(args[opt]);
 			}
 
 			try {
