@@ -95,8 +95,10 @@ public final class Symbol extends Atom implements SymbolName {
 	}
 
 	//
-	/*package*/ static final Namespace DEFAULT_NAMESPACE =
-		new Namespace();
+	static final Namespace DEFAULT_NAMESPACE = new Namespace();
+	static final int NO_FOLD_CASE = 0;
+	static final int UPPER_FOLD_CASE = 1;
+	static final int LOWER_FOLD_CASE = 2;
 
 	//
 	public static final Symbol QUOTE =
@@ -108,6 +110,9 @@ public final class Symbol extends Atom implements SymbolName {
 	public static final Symbol UNQUOTE_SPLICING =
 		DEFAULT_NAMESPACE.getSymbol("unquote-splicing");
 
+	//
+	static int foldCase = NO_FOLD_CASE;
+
 	// ----------------------------------------------------
 	// symbol cannot serialize
 	// ----------------------------------------------------
@@ -163,6 +168,22 @@ public final class Symbol extends Atom implements SymbolName {
 			}
 		}
 		return res;*/
+		switch(foldCase) {
+		case NO_FOLD_CASE:
+			return namespace.getSymbol(name);
+		case LOWER_FOLD_CASE:
+		case UPPER_FOLD_CASE:
+			return namespace.getSymbol(name.toLowerCase());
+		default:  throw new RuntimeException();
+		}
+	}
+
+	/**
+	 * 
+	 * @param name
+	 * @return
+	 */
+	public static Symbol getSymbolWithoutFoldCase(String name) {
 		return namespace.getSymbol(name);
 	}
 
@@ -283,18 +304,49 @@ public final class Symbol extends Atom implements SymbolName {
 		return new LispString(getName());
 	}
 
+	/**
+	 * 
+	 * @return
+	 */
+	public boolean isNormal() {
+		String s = getName();
+
+		for(int i = 0; i < s.length(); i++) {
+			if(LispUtils.isReservedCharacter(s.charAt(i))) {
+				return false;
+			} else if(foldCase != NO_FOLD_CASE &&
+					Character.isUpperCase(s.charAt(i))) {
+				return false;
+			}
+		}
+		return true;
+	}
+
 	/* (non-Javadoc)
 	 * @see net.morilib.lisp.Atom#print()
 	 */
 	public String print() {
-		return getName();
+//		return getName();
+//		return isNormal() ? getName() : "|" + getName() + "|";
+		if(!isNormal()) {
+			return "|" + getName() + "|";
+		} else if(foldCase == NO_FOLD_CASE) {
+			return getName();
+		} else if(foldCase == LOWER_FOLD_CASE) {
+			return getName().toLowerCase();
+		} else if(foldCase == UPPER_FOLD_CASE) {
+			return getName().toUpperCase();
+		} else {
+			throw new RuntimeException();
+		}
 	}
 
 	/* (non-Javadoc)
 	 * @see net.morilib.lisp.Atom#getResult()
 	 */
 	public String getResult() {
-		return getName();
+//		return getName();
+		return print();
 	}
 
 	/*
