@@ -22,6 +22,7 @@ import net.morilib.lisp.LispInteger;
 import net.morilib.lisp.LispMessage;
 import net.morilib.lisp.LispReal;
 import net.morilib.lisp.Undef;
+import net.morilib.lisp.accessor.ILispRef;
 import net.morilib.lisp.subr.BinaryArgs;
 import net.morilib.lisp.subr.SubrUtils;
 import net.morilib.lisp.subr.TernaryArgs;
@@ -32,7 +33,7 @@ import net.morilib.lisp.subr.UnaryArgs;
  *
  * @author MORIGUCHI, Yuichiro 2011/03/06
  */
-public interface HomogeneousArray extends ILispVector {
+public interface HomogeneousArray extends ILispVector, ILispRef {
 
 	/**
 	 * 
