@@ -377,4 +377,17 @@ implements HomogeneousSignedArray, java.io.Serializable {
 		vector.setDouble(index, x);
 	}
 
+	/* (non-Javadoc)
+	 * @see net.morilib.lisp.accessor.ILispRef#ref(net.morilib.lisp.Datum, net.morilib.lisp.LispMessage)
+	 */
+	@Override
+	public Datum ref(Datum arg, LispMessage mesg) {
+		int k = SubrUtils.getSmallInt(arg, mesg);
+
+		if(k < 0 || k >= size()) {
+			throw mesg.getError("err.accessor.ref.outofrange", arg);
+		}
+		return get(k);
+	}
+
 }
